/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * errno.h : error number definitions
 */


#ifndef __ERRNO_H
#define __ERRNO_H
#ifdef __cplusplus
extern "C" {
#endif

#include <sys/errno.h>

/* For single-threaded code, keep the old variable */
extern int errno;

/* Returns a pointer to the current thread's errno variable. */
int * __sdethread_errno_pointer (void) __attribute__ ((__const__));

/* Assembler library module support function: set current thread's
  errno to the first argument, and then return its second argument. */
__attribute__((__nomips16__))
int	__sdethread_set_errno (int, int);

#ifndef _NONREENTRANT

#pragma weak __sdethread_errno_pointer

static int *__errno_pointer (void) __attribute__ ((__const__));

static __inline__ int *
__errno_pointer (void)
{
    /* For threaded code, use the thread-specific pointer, if present;
       else use errno variable. */
    if (!__sdethread_errno_pointer)
	return &errno;
    return __sdethread_errno_pointer ();
}

#define errno (*__errno_pointer ())

#endif /* ! _NONREENTRANT */

#ifdef __cplusplus
}
#endif
#endif /* !defined __ERRNO_H */
