/*
 * Copyright (c) 2005 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * fenv.h : C99 floating point control 
 */

#ifndef __FENV_H
#ifdef __cplusplus
extern "C" {
#endif
#define __FENV_H

#include <machine/fenv.h>

/* Floating point "exception" status */
extern int feclearexcept (int);
extern int fetestexcept (int);
extern int feraiseexcept (int);

/* Floating point "exception" status (another way) */
extern int fegetexceptflag (fexcept_t *, int);
extern int fesetexceptflag (const fexcept_t *, int);

/* Floating point rounding mode */
extern int fegetround (void);
extern int fesetround (int);

/* Floating point state save / restore */
extern int fegetenv (fenv_t *);
extern int feholdexcept (fenv_t *);
extern int fesetenv (const fenv_t *);
extern int feupdateenv (const fenv_t *);

#if !defined(_POSIX_SOURCE) && !defined(_ANSI_SOURCE)
/* Floating point trap control */
extern int feenableexcept (int);
extern int fedisableexcept (int);
extern int fegetexcept (void);
#endif

#ifdef __cplusplus
}
#endif
#endif /* __FENV_H */
