/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * mips/bt.h: Algorithmics rom boot package information
 */


#include <mips/endian.h>

#define BT_NPKG		8	/* number of supported packages */
#define BT_PKGSHIFT	3	/* log2(NPKG) */
#define BT_PKGSIZE	32	/* sizeof(struct btpackage) */
#define BT_MAGIC	((0xcafe << 16)|BYTE_ORDER)
#define BT_FMAGIC	((0x3301 << 16)|BYTE_ORDER)

/* well known packages */
#define BT_PKG_ITROM	6
#define BT_PKG_PMON	7

#ifdef __ASSEMBLER__
#define BT_PACKAGEBASE	PA_TO_KVA1(0x1fc00400)

	.struct 0
bt_magic:	.word	0
bt_start:	.word	0
bt_end:		.word	0
bt_csum:	.word	0
bt_entry:	.word	0
bt_spare:	.word	0:3
	.previous
#else
#define BT_PACKAGEBASE	((struct btpackage *)PA_TO_KVA1(0x1fc00400))
struct btpackage {
	  unsigned int	bt_magic;
	  void		*bt_start;
	  void		*bt_end;
	  unsigned int	bt_sum;
	  unsigned int	bt_entry;
	  unsigned int	:32;
	  unsigned int	:32;
	  unsigned int	:32;
};
#endif

