/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * mqueue.h: POSIX message queue definitions
 */


#ifndef __MQUEUE_H
#define __MQUEUE_H

#include <pthread.h>
#include <sys/signal.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _POSIX_MESSAGE_PASSING

struct mq_attr {
    long		mq_maxmsg;
    long		mq_msgsize;
    long		mq_flags;
    long		mq_curmsgs;
};


struct _mq_opaque;
typedef struct _mq_opaque *mqd_t;

#define MQ_NONBLOCK	0x1

mqd_t	mq_open (const char *, int, ...);
int	mq_close (mqd_t);
int	mq_unlink (const char *);
int	mq_send (mqd_t, const char *, size_t, unsigned int);
int	mq_receive (mqd_t, char *, size_t, unsigned int *);
int	mq_timedreceive (mqd_t, char *, size_t, unsigned int *, 
			 struct timespec *);
int	mq_notify (mqd_t, const struct sigevent *);
int	mq_setattr (mqd_t, const struct mq_attr *, struct mq_attr *);
int	mq_getattr (mqd_t, struct mq_attr *);
#endif /* _POSIX_MESSAGE_PASSING */

#ifdef __cplusplus
}
#endif

#endif /* __MQUEUE_H */
