/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * semaphore.h: POSIX semaphore definitions
 */


#ifndef __SEMAPHORE_H
#define __SEMAPHORE_H

#include <pthread.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _POSIX_SEMAPHORES
typedef struct {
  _pthread_waitq_t	sem_waitq;
  int 	 		sem_value;
#define SEM_VALUE_MAX	INT_MAX
  const char *		sem_name;
} sem_t;

#define SEM_FAILED	((sem_t *) 0)

/* unnamed semaphores */
int sem_init (sem_t *, int, unsigned int);
int sem_destroy (sem_t *);

/* named semaphores */
sem_t * sem_open (const char *, int, ...);
int sem_close (sem_t *);
int sem_unlink (const char *);

/* generic semaphore operations */
int sem_wait (sem_t *);
int sem_trywait (sem_t *);
int sem_post (sem_t *);
int sem_getvalue (sem_t *, int *);

int sem_setname_np (sem_t *, const char *);
int sem_getname_np (sem_t *, const char **);

#define SEM_INITIALIZER(n)	{ { NULL }, n }

#endif /* _POSIX_SEMAPHORES */

#ifdef __cplusplus
}
#endif

#endif /* __SEMAPHORE_H */
