/*
 * Copyright (c) 1996-2007 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * string.h : ANSI string function definitions
 */


#ifndef __STRING_H
#ifdef __cplusplus
extern "C" {
#endif
#define __STRING_H

/* wrong gets offsetof !! 
*/
#include <stddef.h>

void	*memcpy	(void *, const void *, size_t);
void 	*memmove(void *,const void *, size_t);
void	*memccpy (void *, const void *, int, size_t);
char	*strcpy	(char *, const char *);
char	*strncpy (char *, const char *, size_t);
char	*strcat	(char *, const char *);
char	*strncat (char *, const char *, size_t);
int	memcmp	(const void *, const void *, size_t);
int	strcmp	(const char *, const char *);
int	strncmp	(const char *, const char *, size_t);
void	*memchr	(const void *, int, size_t);
char	*strchr	(const char *, int);
size_t	strcspn	(const char *, const char *);
char	*strpbrk (const char *, const char *);
char	*strrchr (const char *, int);
size_t	strspn	(const char *, const char *);
char	*strstr	(const char *, const char *);
char	*strtok	(char *, const char *);
char	*strtok_r (char *, const char *, char **);
void	*memset	(void *, int , size_t);
size_t	strlen	(const char *);

int	strcoll (const char *, const char *);
size_t	strxfrm (char *, const char *, size_t);

#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
/* BSD backwards compatibility */
static __inline void bcopy (const void *_s, void *_d, size_t _n) 
{ memcpy (_d, _s, _n); }
static __inline int bcmp (const void *_a, const void *_b, size_t _n) 
{ return memcmp (_a, _b, _n); }
/* Don't inline bzero, as gcc might use it through the bzero builtin.  */
void bzero (void *_d, size_t _n);
static __inline char * index (const char *_s, int _c) 
{ return strchr (_s, _c); }
static __inline char * rindex (const char *_s, int _c) 
{ return strrchr (_s, _c); }

/* BSD library functions */
char	*strdup (const char *);
int	 strcasecmp (const char *, const char *);
int	 strncasecmp (const char *, const char *, size_t);
char	*strsep (char **, const char *);
size_t	 strlcat (char *, const char *, size_t);
size_t	 strlcpy (char *, const char *, size_t);
int	 ffs (int);
int	 ffsl (long int);
#ifndef __STRICT_ANSI__
int	 ffsll (long long int);
#endif
char	*strsignal (int);
#endif

/*
 * Putting this in here is really silly, but who am I to argue with X3J11?
 */
char *strerror	(int);
char *strerror_r (int, char *, size_t);

#ifdef __cplusplus
}
#endif
#endif /* !defined __STRING_H */
