/*
 * Copyright (c) 1996-2006 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * time.h : ANSII/POSIX time fdefinitions
 */


#ifndef __TIME_H
#ifdef __cplusplus
extern "C" {
#endif
#define __TIME_H

/* This is ANSI C and POSIX compliant
 */

/* --- Inclusions --- */
#include <machine/ansi.h>
#include <stddef.h>

/* --- Types --- */

#ifdef _CLOCK_T_
typedef _CLOCK_T_ clock_t;
#undef _CLOCK_T_
#endif

#ifdef _TIME_T_
typedef _TIME_T_ time_t;
#undef _TIME_T_
#endif

/* --- Structures --- */
struct tm {
	int	tm_sec;
	int	tm_min;
	int	tm_hour;
	int	tm_mday;
	int	tm_mon;
	int	tm_year;
	int	tm_wday;
	int	tm_yday;
	int	tm_isdst;
	/* BSD fields */
	long	tm_gmtoff;
	char	*tm_zone;
};

/* --- Prototypes --- */
time_t	time(time_t *);
clock_t	clock(void);
char	*asctime_r(const struct tm *, char *);
char	*asctime(const struct tm *);
char	*ctime_r(const time_t *, char *);
char	*ctime(const time_t *);
double	difftime(time_t, time_t);
struct tm *gmtime_r(const time_t *, struct tm *);
struct tm *gmtime(const time_t *);
struct tm *localtime_r(const time_t *, struct tm *);
struct tm *localtime(const time_t *);
time_t	mktime(struct tm *);
size_t	strftime(char *, size_t, 
		 const char *, const struct tm *);

/* CLOCKS_PER_SEC very sys dependent 
*/
#include <sys/clock.h>

#if !defined(_ANSI_SOURCE)
void	tzset(void);
extern  char 	*tzname[2];
#endif

#ifdef __cplusplus
}
#endif
#endif /* !defined __TIME_H */
