#ident "kit/LSIPR/sbdclock.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * LSIPR/sbdclock.c: timing support functions for LSI Pocket Rocket
 */


#include <stddef.h>
#include <sys/time.h>
#include <sys/clock.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>
#include <lsi/lr33000.h>
#include <assert.h>

#include "kit_impl.h"
#include "sbd.h"

static clock_t		(*tickfunc)();
static clock_t		tick;
static clock_t		maxtick;
static unsigned long	cycpertick;
static unsigned long	cycperusec;

clock_t			_clk_tck;

void
_sbd_settimer (clock_t newtick)
{
    assert (newtick <= maxtick);
    if (newtick != tick) {
	cycpertick = (tick = newtick) * cycperusec;
	TC1 = 0;		/* stop clock */
	TIC1 = cycpertick;	/* program counter value */
	TC1 = TC_CE | TC_IE;	/* restart clock (XXX accuracy lost) */
    }
}


static int
clkintr (int arg, struct xcptcontext *xcp)
{
    clock_t newtick;

    /* acknowledge timer interrupt */
    TC1 &= ~TC_INT;

    /* call timer tick function */
    newtick = (*tickfunc) (1, xcp);
    
    /* update timer if a new tick interval is required */
    if (newtick != tick)
	_sbd_settimer (newtick);

    return 0;
}


clock_t
_sbd_gettimer ()
{
    /* return how many microseconds have passed since last interrupt */
    unsigned long cycles;

    /* get timer */
    cycles = TIC1;

    /* convert downcount to upcount */
    cycles = (cycpertick - cycles);

    /* check for possible missed interrupt */
    if (cycles < cycpertick / 2 && (TC1 & TC_INT))
      cycles += cycpertick;

    return cycles / cycperusec;
}


clock_t
_sbd_inittimer (clock_t (*func)())
{
    if (!_clk_tck) {
	/* get CPU frequency */
	_clk_tck = _sbd_cpufreq ();
    }

    /* work out timing parameters based on that frequency */
    cycperusec = _clk_tck / 1000000;
    maxtick = 0x1000000 / cycperusec;
    
    /* stop timer (started in settimer) */
    TC1 = 0;

    /* install interrupt handler */
    tickfunc = func;
    intrupt (INTR_TINT1, clkintr, 0);
    mips_bissr (SR_IM2);	/* enable interrupt now */

    return maxtick;
}
