#ident "kit/P4000B/sbddbg.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1996-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * P4000/sbddbg.c: Additional debug support for P4000 board
 */


#include <stddef.h>
#include <signal.h>
#include <mips/cpu.h>
#include <mips/xcpt.h>

#include "sbd.h"
#include "kit_impl.h"

static int
panichandler (int arg, struct xcptcontext *xcp)
{
    unsigned int irr0 = *(volatile unsigned int *) PA_TO_KVA1 (INT_IRR0);
    (void) *(volatile unsigned int *) PA_TO_KVA1 (INT_ACKPANIC);
    if (irr0 & IRR0_BUS)
	return SIGBUS;
    if (irr0 & IRR0_ACFAIL)
	return SIGINT;		/* FIXME ?? */
    if (irr0 & IRR0_DEBUG) {
	/* software debounce debug button
	 * (hope no other panics arrive in the meantime)
	 */
	do {
	    (void) *(volatile unsigned int *) PA_TO_KVA1 (INT_ACKPANIC);
	    /* usdelay (1000); */
	    irr0 = *(volatile unsigned int *) PA_TO_KVA1 (INT_IRR0);
	} while (irr0 & IRR0_DEBUG);
	return SIGTRAP;
    }
    return SIGINT;
}

static int
serhandler (int arg, struct xcptcontext *xcp)
{
    unsigned int irr0 = *(volatile unsigned int *) PA_TO_KVA1 (INT_IRR0);

    if (irr0 & IRR0_SIO) {
	if (_dbg_inpoll () == 0x03) 
	    return (SIGTRAP);
    }
    
    return (0);
}

void
_dbg_signals (void)
{
    unsigned int sr;

    /*
     * intrupt does not enable interrupts so we do them here
     */
    sr = 0;

    /* enable debug interrupt */
    (void) intrupt (6, panichandler, 0);
    sr |= SR_IM6;

    /* enable serial interrupt for debug channel */
    (void) intrupt (2, serhandler, 0);
    sr |= SR_IM2;

    mips_bissr(sr);
}
