#ident "kit/P4000B/sbdnvram.c: Copyright (c) MIPS Technologies, Inc. All rights reserved."

/*
 * Copyright (c) 1997-2003 MIPS Technologies, Inc.
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above
 *      copyright
 *        notice, this list of conditions and the following disclaimer
 *        in the documentation and/or other materials provided with
 *        the distribution.
 *      * Neither the name of MIPS Technologies Inc. nor the names of its
 *        contributors may be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * P4000B/sbdnvram.c: low-level interface to P4000 NVRAM
 */


#include <stddef.h>
#include <mips/cpu.h>

#include "kit_impl.h"
#include "sbd.h"
#include "nvenv.h"
#include "mk48t02.h"

#define NVPOS(x)	(td->td_mem[x])
#define NVRD(x)		(NVPOS(x) >> 24)
#define NVWR(x,v)	NVPOS(x) = ((v)<<24)

unsigned int 
_sbd_nvram_size (unsigned int *nvoffs, unsigned int *nvsize)
{
    if (nvoffs) *nvoffs = ENVOFFSET;
    if (nvsize) *nvsize = TD_NVRAM_SIZE-ENVOFFSET;
    return TD_NVRAM_SIZE;	/* ??? */
}


void
_sbd_nvram_flush (int offs)
{
}


unsigned char
_sbd_nvram_getbyte(unsigned int offs)
{
    register volatile struct td_clock *td =
	(struct td_clock *)PA_TO_KVA1(RTCLOCK_BASE);

    return (NVRD(offs));
}


void
_sbd_nvram_setbyte(unsigned int offs, unsigned char val)
{
    register volatile struct td_clock *td =
	(struct td_clock *)PA_TO_KVA1(RTCLOCK_BASE);

    NVWR(offs, val);
}


/*
 * BigEndian!
 */
unsigned short
_sbd_nvram_getshort (unsigned int offs)
{
    return ((_sbd_nvram_getbyte(offs) << 8) | _sbd_nvram_getbyte(offs+1));
}


void
_sbd_nvram_setshort (unsigned int offs, unsigned short val)
{
    _sbd_nvram_setbyte (offs, (unsigned char)((val >> 8) & 0xff));
    _sbd_nvram_setbyte (offs+1, (unsigned char)(val & 0xff));
}


int
_sbd_nvram_hwtest (void)
{
    unsigned int offs = ENVOFFSET + ENVOFF_TEST;

    _sbd_nvram_setbyte (offs, 0xaa);
    _sbd_nvram_setbyte (offs+1, 0x55);
    if (_sbd_nvram_getbyte (offs) != 0xaa
        || _sbd_nvram_getbyte (offs + 1) != 0x55)
	return 0;

    _sbd_nvram_setbyte (offs, 0x55);
    _sbd_nvram_setbyte (offs + 1, 0xaa);
    if (_sbd_nvram_getbyte (offs) != 0x55
        || _sbd_nvram_getbyte (offs + 1) != 0xaa)
	return 0;

    return 1;
}

