# Copyright (c) 1996-2007 MIPS Technologies, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above
#      copyright
#        notice, this list of conditions and the following disclaimer
#        in the documentation and/or other materials provided with
#        the distribution.
#      * Neither the name of MIPS Technologies Inc. nor the names of its
#        contributors may be used to endorse or promote products derived
#        from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#
# kit/kit.mk
#
# Shared make include file for MIPS SDE embedded system kit


.PHONY: all buildall

all:	buildall

# defaults, which may be overridden by sbd.mk
BAREMON	=bare
CPU	=r3k
CACHES	=$(CP0)
TLB	=$(CP0)
WPOINTS =$(CACHES)
NVENV	=noenv
FPU	=no
PCI	=no
ENDIAN	=-EB
ROM_U	=bfc00000
ROM_C	=9fc00000
ROM_SZ	=400000
RAM_U	=a0000280
RAM_C	=80000280

INCDIRS	=

XCPTLOW = xcptlowb

ifndef FREESDE
-include ../fullsde.mk
endif

KITDIR	=..

# Here we include the board-specific sbd.mk
include sbd.mk

# Pick Cop0 support files - defaulting to "m32" for known MIPS32 CPUs and ISAs
M32CPUs	=4k% 5k% 20k% 24k% 25k% m4k% 34k% 74k%
M32ISAs	=-mips32% -mips64%

ifndef CP0
 ifneq (,$(filter $(M32CPUs),$(CPU))$(filter $(M32ISAs),$(ISA)))
  CP0	=m32
 else
  CP0	=$(CPU)
 endif
endif

ROMDLFMT ?=s3

KITALL	=sastart.o libram.a librdebugram.a librom.a librdebugrom.a crtn.o

ifneq ($(FPU),no)
KITALL += libcp1.a
CP1SRC	=cp1emu.c
CP1OBJ	=cp1emu.o
endif

ifneq (,$(SBDLOW))
KITALL	+=romstart.o micromon.$(ROMDLFMT)
endif

ALL	=kit

LIB_TEST=kit-test
APP_TEST=example-test

.PHONY: kit
kit:	$(KITALL)

ifeq (,$(CACHES))
  CACHEDEF = '-Acache(no)'
  CACHESRC =
else
  CACHEDEF=$(foreach cache,$(CACHES),'-Acache($(cache))')
  CACHESRC=cache.sx cache_ops.sx $(foreach cache,$(CACHES),$(cache)cache.sx $(cache)cache_ops.sx)
  CACHEOBJ=cache.o cache_ops.o $(foreach cache,$(CACHES),$(cache)cache.o $(cache)cache_ops.o)
  CACHEXOBJ=cachex.o $(foreach cache,$(CACHES),$(cache)cachex.o)
endif

ifeq (,$(WPOINTS))
  WATCHDEF = '-Awatch(no)'
  WATCHSRC =
else
  WATCHDEF = $(foreach wp,$(WPOINTS),'-Awatch($(wp))')
  WATCHSRC = watch.c watchsup.c $(foreach wp,$(WPOINTS),$(wp)watch.c)
  WATCHOBJ = watch.o watchsup.o $(foreach wp,$(WPOINTS),$(wp)watch.o)
endif

ifdef FULLSDE
%cachex.o: %cache.sx
	$(AS) $(ASFLAGS) $(CPPFLAGS) -D_SDE_CACHE_EXTRA -D__ASSEMBLER__ -x assembler-with-cpp $< -c -o $@

cachex.o: cache.sx
	$(AS) $(ASFLAGS) $(CPPFLAGS) -D_SDE_CACHE_EXTRA -D__ASSEMBLER__ -x assembler-with-cpp $< -c -o $@
endif

# Select fpu control functions
ifeq ($(FPU),no)
  CP1		=noc1
else
  CP1		=$(CP0)c1
endif

# Set up search path for sources only
_VPATH	=$(SBDDIRS):../$(MONITOR):../$(BAREMON):../share:../pci

##
## Generic kit files
##

KITSRC = xcpt.c xcptshow.c xcptshowmin.c xcptstack.c $(XCPTLOW).sx \
	endian.c initfini.c nsdelay.c $(WATCHSRC) \
	stubs.c posix.c dirops.c getdir.c mfs.c sysconf.c \
	signals.c ecchandler.c intrupt.c timer.c profil.c mon.c \
	m16entry.c muldivem.c bremu.c unaligned.c \
	mipscp0.sx mips16.sx cca.sx clz.c sync.c \
	fcache.c flashdev.c paneldev.c env.c $(NVENV).c \
	$(TLB)tlb.sx $(TLB)tlb_ops.sx $(CACHESRC) $(CP1).sx

KITOBJ = $(filter %.o,$(KITSRC:.c=.o) $(KITSRC:.cc=.o) $(KITSRC:.sx=.o) $(KITSRC:.s=.o) $(KITSRC:.S=.o)) \
	$(CACHEXOBJ)

# List of optional generic features supported in this version of the kit.
KITFEATURES += xcptstackinfo

ifneq ($(PCI),no)
  KITSRC += pciconf.c pcidevs.c pciio.c pcilookup.c
  KITFEATURES += pci pcilookup pciconf
endif

# monitor interfaces for ROM and RAM
include ../$(BAREMON)/monitor.mk
include ../$(MONITOR)/monitor.mk

# Get SDE version number information
-include $(KITDIR)/sdeversion

include ../rules.mk

ifndef SDE_MAJOR
# Guess major SDE version based on compiler version
SDE_MAJOR = $(shell \
	if $(CC) -v 2>&1 | fgrep 'gcc version 4.' >/dev/null; then \
	  echo "7"; \
	elif $(CC) -v 2>&1 | fgrep 'gcc version 3.' >/dev/null; then \
	  echo "6"; \
	else \
	  echo "5"; \
	fi)
endif

# Include some per-architecure specific files and settings
-include $(KITDIR)/archs.mk

ALLOBJS =$(sort $(SBDLOW) $(SBDOBJ) \
	   romlow.o $(KITOBJ) $(CP1OBJ) \
	   $(MONLOW) $(MONOBJ) $(MONDBG) \
	   $(BARELOW) $(BAREOBJ) $(BAREDBG))

ifneq (,$(SBDLOW))
ALLOBJS += micromon.o
endif

# Compute list of object files generated from board-specific source files
#LOCALOBJS=$(filter %.o,$(patsubst %.c,%.o,$(patsubst %.s,%.o,$(patsubst %.sx,%.o,$(SBDSRC)))))

# We no longer supply any board-specific source code in free/eval version
LOCALOBJS=

# Then compute the list of all "shared" objects (i.e. ones not derived
# from board-specific source code).
SHAREDOBJS=$(filter-out $(LOCALOBJS),$(ALLOBJS))

# Define CPU architecture for header files etc.
ifndef CPUASSERT
 ifndef CPUNAMES
  CPUALIAS	=$(patsubst r%k,r%000,$(filter r%k,$(CPU)))
  CPUNAMES	=$(CPU) $(CPUALIAS) $(CPUVARIANT)
 endif
 CPUASSERT	=$(patsubst %,'-Acpu(%)',$(CPUNAMES))
endif

ifneq (,$(filter $(filter -mips%,$(ISA)),-mips3 -mips4 -mips64 -mips64r2))
ifeq (,$(filter $(filter -mgp%,$(ISA)),-mgp32))
ISA_IS_64BIT = 1
endif
endif

ifneq (,$(strip $(filter 7,$(SDE_MAJOR))))
 CPUARCH = -march=$(CPU)
 # ISA was traditionally used to indicate 32-bit/64-bit support.
 # FIXME temporary workaround for SDE 7 ABI selection
 ifeq (,$(ISA_IS_64BIT))
  ISA =
  ABI = -mabi=32
 else
  ISA =
  ABI = -mabi=n32
 endif
endif

# Pick CPU architecture for compiler to tune for
ifndef CPUTUNE
 ifneq (,$(strip $(filter 5,$(SDE_MAJOR))))
  CPUTUNE	=-mcpu=$(CPU)
  CPUTUNE	+=$(patsubst %,-mcpu=%,$(CPUNAMES))
 else
  ifneq (,$(strip $(filter 6,$(SDE_MAJOR))))
   CPUTUNE	=-mtune=$(CPU)
  endif
 endif
endif

CPUFLAGS=$(ABI) $(ISA) $(CPUARCH) $(CPUTUNE) $(ASE) $(ENDIAN) $(CPUASSERT) 

ifneq (,$(strip $(filter r3k,$(CPU))))
 CPUFLAGS += -Wa,-break
endif

INCDIRS+=-I. $(patsubst %,-I%,$(subst :, ,$(SBDDIRS)))
ifneq ($(PCI),no)
INCDIRS+=-I../pci
endif
ifneq (,$(strip $(filter 7,$(SDE_MAJOR))))
INCDIRS+=-iquote../$(MONITOR) -iquote../share $(STDINCDIR)
else
INCDIRS+=-I../$(MONITOR) -I../share -I- $(STDINCDIR)
endif

# define DEBUG=-UNDEBUG to enable assertions in library
DEBUG	?=-DNDEBUG 

ifndef FULLSDE
FREEKIT=-DFREE_SDE
endif

CPPFLAGS += $(SBDFLAGS) $(SBDOPTIONS) $(KITFLAGS) $(DEBUG) $(CACHEDEF) $(WATCHDEF) '-Ammu($(TLB))' $(ARCHFLAGS) $(INCDIRS) $(FREEKIT)

ifneq (,$(strip $(filter -m%,$(FPU))))
CPUFLAGS+=$(FPU)
CPPFLAGS+='-Afpu($(patsubst -m%,%,$(FPU)))'
else
CPPFLAGS+='-Afpu($(FPU))'
endif

# Add board-specific features
CPPFLAGS+=$(foreach feature,$(SBDFEATURES) $(MONFEATURES) $(KITFEATURES),'-Afeature($(feature))')

ifneq ($(PCI),no)
 CPPFLAGS+= '-Apci($(PCI))'
endif

ifneq ($(ROM_U),bfc00000)
 CPPFLAGS+=-DROM_BASE=0x$(ROM_U)
endif
ifneq ($(RAM_U),a0000280)
 CPPFLAGS+=-DRAM_BASE=0x$(RAM_U)
endif

# User may override these options
KITOPTIM=-Os
ifndef FREEBUILD
override CFLAGS	=$(KITOPTIM) -g $(ASE) $(KITCFLAGS) -ffunction-sections -D_FUNCTION_SECTIONS
else
# No debug info in free libs 
override CFLAGS	=$(KITOPTIM) $(ASE) $(KITCFLAGS) -ffunction-sections -D_FUNCTION_SECTIONS
endif
override CFLAGS += $(CSL_CFLAGS)
override ASFLAGS := $(CFLAGS)

override WFLAGS	=-Wall -Wno-parentheses -Wno-uninitialized -Wmissing-prototypes -Wno-inline -Werror

# But not these
override CFLAGS	+= -fno-optimize-sibling-calls $(WFLAGS) $(CPUFLAGS)
override ASFLAGS += -fno-optimize-sibling-calls $(WFLAGS) $(CPUFLAGS)

ifneq (,$(strip $(filter 7,$(SDE_MAJOR))))
# FIXME temporary workaround for SDE 7
ifeq (,$(ISA_IS_64BIT))
ifeq (-EL,$(strip $(ENDIAN)))
override LD_OFORMAT='-Wl,--oformat=elf32-tradlittlemips'
else
override LD_OFORMAT='-Wl,--oformat=elf32-tradbigmips'
endif
else
ifeq (-EL,$(strip $(ENDIAN)))
override LD_OFORMAT='-Wl,--oformat=elf32-ntradlittlemips'
else
override LD_OFORMAT='-Wl,--oformat=elf32-ntradbigmips'
endif
endif
endif

ifneq (,$(strip $(filter 5,$(SDE_MAJOR))))
override CFLAGS +=-mno-gpopt 
override LDFLAGS+=$(ENDIAN) $(LD_OFORMAT)
else
override CFLAGS +=-G 0
override ASFLAGS +=-G 0
override LDFLAGS +=-G 0 $(ENDIAN) $(LD_OFORMAT) $(ISA) $(ASE)
endif

ifdef LIBRARY_PATH
 # split LIBRARY_PATH into separate -L flags for linker
 override LDFLAGS += $(patsubst %,-L%,$(subst :, ,$(LIBRARY_PATH)))
endif

# These objects are startup code which is "position-independent", and
# uses "bal" instead "jal" instructions, so they must be within 128K of 
# each other.

# modules that MUST come first in a ROM program
romstart.o: romlow.o
	$(CC) $(LDFLAGS) -Wl,-r -Wl,--script=$(KITDIR)/share/sde-r.ld -nostdlib -o $@ $^

# modules that MUST come first in a standalone downloadable program
sastart.o: $(MONLOW)
	$(CC) $(LDFLAGS) -Wl,-r -Wl,--script=$(KITDIR)/share/sde-r.ld -nostdlib -o $@ $^

librom.a: $(KITOBJ) $(SBDOBJ) $(BAREOBJ) $(SBDLOW) $(BARELOW)
	$(AR) $(ARFLAGS) $@ $?

librdebugrom.a: $(BAREDBG)
	$(AR) $(ARFLAGS) $@ $?

libram.a: $(KITOBJ) $(SBDOBJ) $(MONOBJ) $(MONLOW)
	$(AR) $(ARFLAGS) $@ $?

librdebugram.a: $(MONDBG)
	$(AR) $(ARFLAGS) $@ $?

libcp1.a: $(CP1OBJ)
	$(AR) $(ARFLAGS) $@ $?

crtn.o: crtn.sx

ifdef FULLSDE
$(BARELOW) $(BAREOBJ) $(BAREDBG): ../$(BAREMON)/monitor.mk
$(MONLOW) $(MONOBJ) $(MONDBG): ../$(MONITOR)/monitor.mk

ifneq (,$(strip $(filter 6,$(SDE_MAJOR))))
flashrom.o: flashrom.c
	$(CC) -c $(CPPFLAGS) $(CFLAGS) -fno-unit-at-a-time $< -o $@
endif
endif

# Select the correct arguments for sde-conv when making a ROM
ifeq ($(ROM_U)/$(ROM_SZ),bfc00000/400000)
  ROMFLG=-p
else
  ROMFLG=-a $(ROM_U)/$(ROM_SZ)
endif

micromon.$(ROMDLFMT): micromon
	$(CONV) -vf $(ROMDLFMT) $(ROMFLG) -o micromon.$(ROMDLFMT) micromon

ifneq (,$(SBDLOW))
micromon: micromon.o $(SBDLOW) $(TLB)tlb.o $(CACHEOBJ)
	$(CC) $(LDFLAGS) -Wl,--script=$(KITDIR)/share/sde.ld \
		-nostartfiles -nostdlib -Ttext $(ROM_U) -Tdata $(RAM_U) -o $@ $^
else
micromon: 
	@echo "Cannot build micromon without SBDLOW defined"
endif

#$(ALLOBJS): Makefile sbd.mk ../kit.mk
#Makefile sbd.mk ../kit.mk:

CLEAN = kitclean

# Subsidiary per-target library makefiles
-include ../../rtpxlite/rtpxlite.mk
-include ../../rtpx/rtpx.mk
-include ../../net/net.mk

ifeq (,$(TXROOTDIR))
-include ../../threadx/threadx.mk
else
-include $(TXROOTDIR)/threadx.mk
endif

buildall: $(ALL) # reminder

.PHONY: clean test kitclean clobber reallyclean

clean: $(CLEAN)

clobber reallyclean: clean
	rm -f *.pco *.d *[#~]
	rm -f $(LIB_TEST) $(APP_TEST)

.PHONY: test lib-test app-test
test:
	@$(MAKE) SBD=$(SBD) lib-test 
	@$(MAKE) SBD=$(SBD) app-test
	@$(MAKE) SBD=$(SBD) clean

lib-test: $(LIB_TEST)

app-test: $(APP_TEST)

kit-test: 
	@$(MAKE) SBD=$(SBD) kitclean
	@$(MAKE) SBD=$(SBD) kit
	@touch $@

example-test: minimon-test tlbxcpt-test paranoia-test whetstone-test
	@touch $@

.PHONY: minimon-test tlbxcpt-test paranoia-test whetstone-test
minimon-test:
	@$(MAKE) -C ../../examples/minimon SBD=$(SBD) KITBUILT=y

tlbxcpt-test:
	@$(MAKE) -C ../../examples/tlbxcpt SBD=$(SBD) KITBUILT=y

paranoia-test:
	@$(MAKE) -C ../../examples/paranoia SBD=$(SBD) KITBUILT=y

whetstone-test:
	@$(MAKE) -C ../../examples/whetstone SBD=$(SBD) KITBUILT=y

kitclean:
	rm -f micromon micromon.$(ROMDLFMT) *.o *.a core *.core

SRCS += romlow.sx $(KITSRC) $(CP1SRC) $(SBDSRC) $(BARESRC) $(MONSRC)

.PHONY: TAGS
TAGS: $(SRCS)
	etags $?

vpath	%.c		$(_VPATH)
vpath	%.cc		$(_VPATH)
vpath	%.s		$(_VPATH)
vpath	%.sx		$(_VPATH)
vpath	%.S		$(_VPATH)
vpath	%.h		$(_VPATH)
vpath	Makefile	$(_VPATH)

PCOLIB	=../free/$(SBD).lib

.PRECIOUS: $(PCOLIB)

.PHONY: reminder

# List of precompiled object files
ifndef FULLSDE

# Extract pre-compiled .o files from pco library
$(SHAREDOBJS): $(PCOLIB)
	$(AR) x $? $@

$(PCOLIB):
	@if [ ! -e $(PCOLIB) ]; then \
	  echo '********************************************************'; \
	  if [ -e .notfree ]; then \
	    echo "$(SBD) target not supported in the free version of SDE."; \
	    echo "If can only be built with the supported SDE package."; \
	  else \
	    echo "$(SBD) target library not found. Check that you have"; \
	    echo "installed the appropriate target board library package."; \
	  fi; \
	  echo '********************************************************'; \
	  exit 1; \
	fi

# Yes it is intentionally easy to remove this nag message!
reminder:
	@echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
	@echo "+++      Building with MIPS Technologies' SDE lite       +++"
	@echo "+++ For a supported and more configurable product visit  +++"
	@echo "+++        http://www.mips.com/products/sde.html         +++"
	@echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"

else

# Build pco library from .o files
.PHONY: free
free: $(ALLOBJS)
	rm -f $(PCOLIB)
	$(AR) $(ARFLAGS) $(PCOLIB) $?
	rm -f $(ALLOBJS)

# Extract pre-compiled .o files from pco library
$(NDAOBJ): $(PCOLIB)
	$(AR) x $? $@

reminder:

# Special assembler dependency
$(XCPTLOW).o: xcptcache.s

endif


# Obsolete dummy depend target, this is now handled automagically.
.PHONY: depend
depend:

# Those rules MUST remain at the end of the whole makefile inclusion
# chain.
%.d: %.c $(NETHDRS) netobj/kernel/machine
	@$(CC) -MM -MG $(CPPFLAGS) $(CFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.cc $(NETHDRS) netobj/kernel/machine
	@$(CC) -MM -MG $(CPPFLAGS) $(CXXFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.sx $(NETHDRS) netobj/kernel/machine
	@$(CC) -MM -MG $(CPPFLAGS) $(ASFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

%.d: %.S $(NETHDRS) netobj/kernel/machine
	@$(CC) -MM -MG $(CPPFLAGS) $(ASFLAGS) $< > $@.$$$$; \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' <$@.$$$$ > $@; \
	rm -f $@.$$$$

ifeq (,$(filter $(MAKECMDGOALS),clean clobber reallyclean))
-include $(filter %.d,$(SRCS:.c=.d) $(SRCS:.cc=.d) $(SRCS:.sx=.d) $(SRCS:.S=.d))
endif
